/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import benten.cat.tm.core.BentenTmEngine;
import benten.cat.tm.core.BentenTmSearchResult;
import benten.cat.tm.engine.AbstractBentenTmEnginePlugin;
import benten.core.text.Strings;
import benten.ui.AbstractBentenUiPlugin;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * CAT TM UI プラグインです。
 */
public class CatTmUiPlugin extends AbstractBentenUiPlugin {

	/** 共用インスタンス */
	private static CatTmUiPlugin plugin;

	private AbstractBentenTmEnginePlugin fTmEnginePlugin = null;
	private final Map<String, AbstractBentenTmEnginePlugin> fTmMap = new HashMap<String, AbstractBentenTmEnginePlugin>();

	/**
	 * コンストラクター。
	 */
	public CatTmUiPlugin() {
	}

	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();
	}

	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * このプラグインの共用インスタンスを取得。
	 * @return このプラグインの共用インスタンス
	 */
	public static CatTmUiPlugin getDefault() {
		return plugin;
	}

	public void loadExtensions() {
		final AbstractBentenTmEnginePlugin tm = getTmEnginePlugin();

		if (tm != null) {
			fTmEnginePlugin = tm;
			try {
				fTmEnginePlugin.load();
			} catch (final Exception e) {
				final UiStatus s = new UiStatus(IStatus.ERROR, e.getMessage()
						+ "\n翻訳 > TM 設定 > 使用する TMX ロケーション を確認してください。");
				UiPlugin.openDialog("TM のロードに失敗しました", s);
			}
		}
	}
	public List<BentenTmSearchResult> fuzzyMatch(String string) {
		synchronized (this) {
			if (fTmEnginePlugin == null) {
				log("fuzzyMatch:TMエンジンがロードされていません。");
				return new ArrayList<BentenTmSearchResult>();
			}
			final IPreferenceStore store = BentenPreference.getStore();
			final boolean ignoreWhitespace = store.getBoolean(Preference.IGNORE_WHITESPACE_TM_REFERENCE.name());
			if (ignoreWhitespace) {
				string = Strings.removeRedundantWhitespace(string);
			}
			final BentenTmEngine tmEngine = fTmEnginePlugin.getEngine();
			return tmEngine.fuzzySearch(string);
		}
	}

	/**
	 * このプラグインが保持している TM エンジンを取得します。
	 *
	 * @return TM エンジン
	 */
	public BentenTmEngine getTmEngine() {
		return fTmEnginePlugin.getEngine();
	}

	/**
	 * TM エンジンのインスタンスを新規に作成します。
	 *
	 * 所定のルールに従って拡張ポイントを検索して TM エンジンのインスタンスを作成します。
	 *
	 * @return TM エンジンのインスタンス。
	 */
	public AbstractBentenTmEnginePlugin getTmEnginePlugin() {
		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.tm.engine"); //$NON-NLS-1$
		if (point == null) {
			return null;
		}

		// このシステムに登録されている、すべての翻訳メモリー・エンジンの一覧を取得。
		// 取得された翻訳メモリー･エンジンの名前を記憶。
		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					final Object helloObj = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (helloObj instanceof AbstractBentenTmEnginePlugin) {
						fTmMap.put(name, (AbstractBentenTmEnginePlugin) helloObj);
					}
				} catch (final CoreException e) {
					log(e);
				}
			}
		}

		// 「設定」から翻訳メモリー･エンジンの名称を取得。
		// 該当する翻訳メモリー・エンジンの翻訳メモリーをロードする。
		final IPreferenceStore store = BentenPreference.getStore();
		final String tmName = store.getString(Preference.TM_NAME.name());
		final AbstractBentenTmEnginePlugin tm = fTmMap.get(tmName);
		return tm;
	}

	/**
	 * TM 名のセットを取得します。
	 * @return TM 名のセット
	 */
	public Set<String> getTmNameSet() {
		return fTmMap.keySet();
	}
}
