/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.core.model;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import benten.core.BentenConstants;
import blanco.tmx.BlancoTmxParser;
import blanco.tmx.BlancoTmxSerializer;
import blanco.tmx.valueobject.BlancoTmx;
import blanco.tmx.valueobject.BlancoTmxTu;

/**
 * TMX モデルとファイルを紐付けて保持するクラス。
 * @author IGA Toshiki
 */
public class BentenTmx {

	/**
	 * 指定されたファイルを BentenTmx としてロード。
	 * @param file ファイル
	 * @return BentenTmx
	 */
	public static BentenTmx loadInstance(File file) {
		return new BentenTmx(file, new BlancoTmxParser().parse(file));
	}

	/**
	 * 指定されたファイルの新規 BentenTmx インスタンスを作成。
	 * @param file ファイル
	 * @return BentenTmx
	 */
	public static BentenTmx newInstance(File file) {
		return new BentenTmx(file, new BlancoTmx());
	}

	private final File file;
	private final BlancoTmx blancoTmx;

	private BentenTmx(File file, BlancoTmx blancoTmx) {
		this.file = file;
		this.blancoTmx = blancoTmx;

		// デフォルト値にプロジェクト名やバージョンを設定します。
		blancoTmx.getHeader().setCreationtool(BentenConstants.PROJECT_NAME_LOWER);
		blancoTmx.getHeader().setCreationtoolversion(BentenConstants.VERSION);
	}

	/**
	 * BlancoTmxTu リストの取得。
	 * @return BlancoTmxTu リスト
	 */
	public List<BlancoTmxTu> getTuList() {
		return blancoTmx.getBody().getTuList();
	}

	/**
	 * BlancoTmxTu の作成。<br>
	 * 作成された BlancoTmxTu はこのインスタンスに追加されます。
	 * @return BlancoTmxTu
	 */
	public BlancoTmxTu createBlancoTmxTu() {
		final BlancoTmxTu tu = new BlancoTmxTu();
		getTuList().add(tu);
		return tu;
	}

	/**
	 * ファイルへ保管。
	 * @throws IOException 入出力例外が発生した場合
	 */
	public void save() throws IOException {
		final List<BlancoTmxTu> newFileList = new LinkedList<BlancoTmxTu>();
		for (final BlancoTmxTu tu : getTuList()) {
			if (tu.getTuvList().size() > 0) {
				newFileList.add(tu);
			}
		}
		if (newFileList.size() > 0) {
			blancoTmx.getBody().setTuList(newFileList);
			new BlancoTmxSerializer().serialize(blancoTmx, file);
		}
	}
}
