/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;

import benten.cat.tm.core.BentenTmDriverManager;
import benten.cat.tm.core.BentenTmEngine;
import benten.cat.tm.core.BentenTmSearchResult;
import benten.cat.tm.ui.CatTmUiPlugin;
import benten.core.BentenConstants;
import benten.core.text.Strings;
import benten.twa.cat.core.valueobject.BentenApplyFuzzyMatchProcessInput;
import benten.twa.cat.messages.BentenApplyFuzzyMatchMessages;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import blanco.commons.util.BlancoFileUtil;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.BlancoXliffUtil;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffAltTrans;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTool;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * 類似訳の適用
 * 
 * <pre>
 * このウィザードは、TMX の類似訳を XLIFF に適用します。
 *   1.  翻訳単位と類似する TMX を探し、翻訳単位の代替翻訳に適用します。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳中間形式機械支援翻訳機能: 類似訳適用機能」に対応します。
 */
public class BentenApplyFuzzyMatchProcessImpl extends AbstractTraverseDir implements BentenApplyFuzzyMatchProcess {
	/**
	 * 類似訳適用機能のためのメッセージ。
	 */
	protected static final BentenApplyFuzzyMatchMessages fMsg = new BentenApplyFuzzyMatchMessages();

	protected BentenTmEngine fTm;

	protected BentenApplyFuzzyMatchProcessInput fInput;

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	public void setInput(final BentenApplyFuzzyMatchProcessInput input) {
		fInput = input;
	}

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenApplyFuzzyMatchProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		// トータル件数カウント。
		class FileCounter extends BentenApplyFuzzyMatchProcessImpl {
			private int fCounter = 0;

			@Override
			public void processFile(File htmlFile, String baseDir) {
				fCounter += 2;
			}

			@Override
			protected void processFilterdFile(final File file, final String baseDir) throws IOException {
			}

			public int getCounter() {
				return fCounter;
			}
		}
		final FileCounter inner = new FileCounter();
		inner.setInput(fInput);
		inner.processDir(new File(input.getXliffdir()));
		beginTask(inner.getCounter() + 4);

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		if (fInput.getTmdriverclassname() == null || fInput.getTmdriverclassname().trim().length() == 0) {
			// プラグインの場合のコース。
			fTm = CatTmUiPlugin.getDefault().getTmEnginePlugin().getDriver().getEngineInstance();
		} else {
			// Ant タスクの場合のコース。
			try {
				Class.forName(fInput.getTmdriverclassname());
			} catch (ClassNotFoundException e) {
				throw new IOException(fMsg.getCoreE001(fInput.getTmdriverclassname(), e.toString()));
			}
			if (BentenTmDriverManager.getDrivers().length == 0) {
				throw new IllegalArgumentException(fMsg.getCoreE002());
			}
			fTm = BentenTmDriverManager.getDrivers()[0].getEngineInstance();
		}
		if (fTm == null) {
			throw new IOException(fMsg.getCoreE003());
		}

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		fTm.loadTmx(new File(input.getTmxdir()));

		if (progress(fMsg.getCoreP004())) {
			return 6;
		}

		processDir(new File(input.getXliffdir()));

		if (progress(fMsg.getCoreP005())) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	/**
	 * 処理の内部処理でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progressInner(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	protected void processFile(final File targetXliffFile, final String baseDir) throws IOException {
		if (progress(fMsg.getCoreP011(targetXliffFile.getName()))) {
			return;
		}

		final BlancoXliffParser parser = new BlancoXliffParser();

		BlancoXliff xliff = null;
		try {
			xliff = parser.parse(targetXliffFile);
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE011(targetXliffFile.getName(), e.toString()));
		}

		int transUnitTotalCount = 0;
		for (final BlancoXliffFile file : xliff.getFileList()) {
			transUnitTotalCount += file.getBody().getTransUnitList().size();
		}

		int transUnitCount = 0;
		for (final BlancoXliffFile file : xliff.getFileList()) {
			// 有無を言わさず tool-id を設定します。
			setToolId(file);

			for (final BlancoXliffTransUnit transUnit : file.getBody().getTransUnitList()) {
				if (progressInner(fMsg.getCoreP101(targetXliffFile.getName(), BigDecimal.valueOf(++transUnitCount),
						BigDecimal.valueOf(transUnitTotalCount)))) {
					break;
				}

				if (transUnit.getTranslate() == false) {
					continue;
				}
				if (transUnit.getSource() == null || transUnit.getSource().length() == 0) {
					continue;
				}

				// 自分が設定した alt-trans については、一旦クリアーします。
				clearSelfAltTrans(transUnit);

				// TODO 件数を Eclipse に報告したい。
				processTransUnit(transUnit, fTm);
			}
		}

		if (progressInner(fMsg.getCoreP012(targetXliffFile.getName()))) {
			return;
		}

		try {
			final ByteArrayOutputStream outStream = new ByteArrayOutputStream();
			new BlancoXliffSerializer().serialize(xliff, outStream);
			outStream.flush();

			if (BlancoFileUtil.bytes2FileIfNecessary(outStream.toByteArray(), targetXliffFile) == 0) {
				// 変更無し。
				getResultInfo().setSkipCount(getResultInfo().getSkipCount() + 1);
			} else {
				// 更新のあったファイル数をカウントします。
				getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
			}
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE012(targetXliffFile.getName(), e.toString()));
		}

		if (progress(fMsg.getCoreP013(targetXliffFile.getName()))) {
			return;
		}
	}

	/**
	 * trans-unit を処理します。
	 *
	 * @param transUnit
	 * @param tm
	 */
	protected void processTransUnit(final BlancoXliffTransUnit transUnit, final BentenTmEngine tm) {

		// ホワイトスペースの除去
		String source = transUnit.getSource();
		if (fInput.getIgnorewhitespacetmreference()) {
			source = Strings.removeRedundantWhitespace(source);
		}

		// あいまい翻訳を検索
		final List<BentenTmSearchResult> matchList = tm.fuzzySearch(source);
		if (matchList.size() > 0) {
			fResultInfo.setUnitCount(fResultInfo.getUnitCount() + 1);
		}
		for (final BentenTmSearchResult entry : matchList) {
			final BlancoXliffAltTrans altTrans = new BlancoXliffAltTrans();
			transUnit.getAltTransList().add(altTrans);

			altTrans.setToolId(BentenConstants.PROJECT_NAME_LOWER);
			altTrans.setOrigin(entry.getOrigin());
			altTrans.setMatchQuality(entry.getMatchQuality().trim());
			altTrans.setSource(entry.getSource());
			final BlancoXliffTarget target = new BlancoXliffTarget();
			altTrans.setTarget(target);
			altTrans.getTarget().setTarget(entry.getTarget());
		}
	}

	/**
	 * 自分で登録した alt-trans をクリアーします。
	 *
	 * @param transUnit
	 */
	protected void clearSelfAltTrans(final BlancoXliffTransUnit transUnit) {
		for (int index = transUnit.getAltTransList().size() - 1; index >= 0; index--) {
			final BlancoXliffAltTrans altTrans = transUnit.getAltTransList().get(index);
			if (altTrans.getToolId().equals(BentenConstants.PROJECT_NAME_LOWER)) {
				// これは自分の訳です。
				// 除去してしまいます。
				transUnit.getAltTransList().remove(index);
			}
		}
	}

	/**
	 * tool-id を設定します。
	 *
	 * @param file
	 */
	protected void setToolId(final BlancoXliffFile file) {
		final BlancoXliffTool tool = new BlancoXliffTool();
		tool.setToolId(BentenConstants.PROJECT_NAME_LOWER);
		tool.setToolName(BentenConstants.PROJECT_NAME);
		BlancoXliffUtil.setTool(file.getHeader(), tool);
	}
}
