/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.core;

import java.io.File;
import java.io.IOException;

import org.xml.sax.SAXException;

import benten.core.io.Files;
import benten.twa.filter.core.valueobject.BentenImportHtmlProcessInput;
import benten.twa.filter.messages.BentenImportHtmlMessages;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import blanco.html.normalizer.util.BlancoHtmlNormalizerUtil;
import blanco.html.parser.util.BlancoHtmlParserUtil;

/**
 * 翻訳対象のインポート
 *
 * <pre>
 * 翻訳対象物をプロジェクトにインポートします。
 *   1.  翻訳対象物のファイルをプロジェクトにインポートします。
 *   2.  インポートされた翻訳対象物は、デフォルトではプロジェクト内の [.benten/source] ディレクトリ以下に配置されます。
 *   3.  インポートされた翻訳対象物は、エクスポートの際に利用されます。
 *   4.  インポートされた翻訳対象物は、直接編集しないでください。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳対象物－翻訳中間形式変換機能: HTMLインポート機能」に対応します。
 */
public class BentenImportHtmlProcessImpl extends AbstractTraverseDir implements BentenImportHtmlProcess {
	/**
	 * HTMLインポート機能のためのメッセージ。
	 */
	protected static final BentenImportHtmlMessages fMsg = new BentenImportHtmlMessages();

	protected BentenImportHtmlProcessInput fInput;

	public void setInput(BentenImportHtmlProcessInput input) {
		fInput = input;
	}

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	public int execute(final BentenImportHtmlProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		// トータル件数カウント。
		class FileCounter extends BentenImportHtmlProcessImpl {
			private int fCounter = 0;

			@Override
			public void processFile(File htmlFile, String baseDir) throws IOException {
				fCounter++;
			}

			@Override
			protected void processFilterdFile(final File file, final String baseDir) throws IOException {
				fCounter++;
			}

			public int getCounter() {
				return fCounter;
			}
		}
		final FileCounter inner = new FileCounter();
		inner.setInput(input);
		inner.processDir(new File(input.getSourcedir()));
		beginTask(inner.getCounter());

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		processDir(new File(input.getSourcedir()));

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		return 0;
	}

	public boolean progress(String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(File file) {
		return file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase().endsWith(".htm"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	@Override
	public void processFile(File htmlFile, String baseDir) throws IOException {
		if (progress(fMsg.getCoreP011(htmlFile.getName()))) {
			return;
		}

		byte[] bytes = Files.readFileToByteArray(htmlFile);
		if (fInput.getNormalizehtml()) {
			bytes = BlancoHtmlNormalizerUtil.normalize(bytes);
		}
		final String relativePath = Files.relativePath(new File(fInput.getSourcedir()), htmlFile);
		final File outputHtmlFile = new File(fInput.getTargetdir(), relativePath);
		outputHtmlFile.getParentFile().mkdirs();
		bytes = resolveContentType(bytes);

		Files.writeByteArrayToFile(outputHtmlFile, bytes);

		getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
	}

	@Override
	protected void processFilterdFile(final File file, final String baseDir) throws IOException {
		if (progress(fMsg.getCoreP012(file.getName()))) {
			return;
		}

		final String relativePath = Files.relativePath(new File(fInput.getSourcedir()), file);
		final File outputFile = new File(fInput.getTargetdir(), relativePath);
		Files.copyFile(file, outputFile);

		getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
	}

	private byte[] resolveContentType(byte[] bytes) throws IOException {
		try {
			final String encoding = BlancoHtmlParserUtil.decideEncoding(bytes);
			final String html = new String(bytes, encoding);

			if (!html.matches("(?i)(?s)<meta\\s+http-equiv=\"Content-Type\"\\s+content=\"text/html;\\s+charset=.+?\">")) { //$NON-NLS-1$

				final StringBuilder metaTag = new StringBuilder();
				metaTag.append("\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset="); //$NON-NLS-1$
				metaTag.append(encoding);
				metaTag.append("\">\n"); //$NON-NLS-1$

				String resultHtml = insertMetaTag(html, "<head>", metaTag.toString()); //$NON-NLS-1$
				if (resultHtml == null) {
					resultHtml = insertMetaTag(html, "<html>", metaTag.toString()); //$NON-NLS-1$
				}
				bytes = resultHtml.getBytes(encoding);
			}
		} catch (final SAXException e) {
		}
		return bytes;
	}

	private String insertMetaTag(String html, String beforeTag, String metaTag) {
		final StringBuilder sb = new StringBuilder();
		final int headIndex = html.toLowerCase().indexOf(beforeTag.toLowerCase());
		if (headIndex != -1) {
			sb.append(html.substring(0, headIndex + beforeTag.length()));
			sb.append(metaTag);
			sb.append(html.substring(headIndex + beforeTag.length()));
			return sb.toString();
		}
		return null;
	}
}
