/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.core;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;

import benten.core.BentenConstants;
import benten.core.model.BentenXliff;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import benten.twa.xliff.core.valueobject.BentenMergeXliffProcessInput;
import benten.twa.xliff.messages.BentenMergeXliffMessages;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * XLIFF フラグメントのマージ
 * 
 * <pre>
 * XLIFF フラグメントを、もとになった XLIFF にマージします。
 *   1.  翻訳単位の id をもとに、XLIFF をマージします。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳中間形式断片抽出・マージ機能: XLIFFフラグメント・マージ機能」に対応します。
 */
public class BentenMergeXliffProcessImpl extends AbstractTraverseDir implements BentenMergeXliffProcess {
	/**
	 * XLIFF マージ機能のためのメッセージ。
	 */
	protected static final BentenMergeXliffMessages fMsg = new BentenMergeXliffMessages();

	protected BentenMergeXliffProcessInput fInput;

	public void setInput(BentenMergeXliffProcessInput input) {
		fInput = input;
	}

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * クラスをインスタンス化して処理を実行する際のエントリポイントです。
	 *
	 * @param input 処理の入力パラメータ。
	 * @return 処理の実行結果。
	 * @throws IOException 入出力例外が発生した場合。
	 * @throws IllegalArgumentException 入力値に不正が見つかった場合。
	 */
	public int execute(final BentenMergeXliffProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		// トータル件数カウント。
		class FileCounter extends BentenMergeXliffProcessImpl {
			private int fCounter = 0;

			@Override
			public void processFile(final File inputFile, final String baseDir) throws IOException {
				final FileCounter2 inner2 = new FileCounter2();
				inner2.execute(new File(fInput.getTargetdir()));
				fCounter += inner2.getCounter();
			}

			@Override
			protected void processFilterdFile(final File file, final String baseDir) throws IOException {
				// こちらはカウントしません。
			}

			public int getCounter() {
				return fCounter;
			}
		}
		final FileCounter inner = new FileCounter();
		inner.setInput(input);
		inner.processDir(new File(input.getSourcedir()));
		beginTask(inner.getCounter());

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		processDir(new File(input.getSourcedir()));

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	/**
	 * 処理の内部処理でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progressInner(String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	public void processFile(final File fromXliffFile, final String baseDir) throws IOException {
		if (progress(fMsg.getCoreP011(fromXliffFile.getName()))) {
			return;
		}

		// TODO マージ機能の強化。

		final BentenXliff inXliff = BentenXliff.loadInstance(fromXliffFile);

		class MyTraverseDir extends AbstractTraverseDir {
			BentenXliff fXliff = null;

			boolean fIsSaved = false;

			public void execute(final BentenXliff toXliff, final File dirTo) throws IOException {
				fXliff = toXliff;
				processDir(dirTo);
			}

			@Override
			protected boolean canProcess(final File file) {
				return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
			}

			public boolean isSaved() {
				return fIsSaved;
			}

			@Override
			protected void processFile(File toXliffFile, String baseDir) throws IOException {
				final BentenXliff toXliff = BentenXliff.loadInstance(toXliffFile);
				int transUnitCount = 0;
				int transUnitTotalCount = fXliff.getAllTransUnitList().size();
				for (BlancoXliffTransUnit inUnit : fXliff.getAllTransUnitList()) {
					if (progressInner(fMsg.getCoreP101(toXliffFile.getName(), BigDecimal.valueOf(++transUnitCount),
							BigDecimal.valueOf(transUnitTotalCount)))) {
						break;
					}

					final BlancoXliffTransUnit toUnit = toXliff.getTransUnit(inUnit.getId());

					if (toUnit == null) {
						//System.out.println("id[" + inUnit.getId() + "] は [" + toXliffFile.getName() + "] に見つからず");
						continue;
					}

					if (inUnit.getSource().equals(toUnit.getSource()) == false) {
						// id は一致するのに、source が一致しない場合、上書きしたらまずい。
						continue;
					}

					fResultInfo.setUnitCount(fResultInfo.getUnitCount() + 1);

					// TODO 単に上書きしてしまっているが、これが妥当かどうか検討したい。
					// TODO たとえば、state で優先順位があり、強い方が勝つ、など。
					// TODO また、この動作と過去訳適用の動作とが非常に似ている。

					// TODO 過去訳適用との差違を検討しなおすこと。
					// TODO 少なくとも、過去訳適用の差違には alt-trans に "prev" で過去訳を保持する。

					toUnit.setTranslate(inUnit.getTranslate());
					toUnit.setContextGroupList(inUnit.getContextGroupList());
					toUnit.setTarget(inUnit.getTarget());
					toUnit.setNoteList(inUnit.getNoteList());
				}

				try {
					if (toXliff.save()) {
						fIsSaved = true;
					}
				} catch (IOException e) {
					e.printStackTrace();
					throw new IllegalArgumentException("処理中に例外発生", e);
				}

				if (fIsSaved) {
					if (progress(fMsg.getCoreP012())) {
						return;
					}
				} else {
					if (progress(fMsg.getCoreP013())) {
						return;
					}
				}
			}
		}
		final MyTraverseDir traverseDir = new MyTraverseDir();
		traverseDir.execute(inXliff, new File(fInput.getTargetdir()));
		if (traverseDir.isSaved()) {
			getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
		} else {
			getResultInfo().setSkipCount(getResultInfo().getSkipCount() + 1);
		}
	}
}

class FileCounter2 extends AbstractTraverseDir {
	private int fCounter = 0;

	public void execute(final File target) throws IOException {
		processDir(target);
	}

	public int getCounter() {
		return fCounter;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	protected void processFile(final File outputFile, final String baseDir) throws IOException {
		fCounter++;
	}
}
