/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.jobs;

import java.io.IOException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;
import org.xml.sax.SAXException;

import benten.twa.process.BentenProcessResultInfo;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.ui.UiPlugin;

/**
 * 翻訳ワークフローで使用するファイル・ハンドラー抽象クラスです。
 */
public abstract class AbstractWorkflowFileHandler extends AbstractHandler implements IHandler {

	/** ジョブ・メッセージ */
	protected String jobMessage;

	/** 進捗モニター */
	protected IProgressMonitor monitor = new NullProgressMonitor();

	/** 処理結果情報 */
	protected BentenProcessResultInfo resultInfo;

	/**
	 * ジョブ・メッセージをセットします。
	 * @param jobMessage ジョブ・メッセージ
	 */
	protected void setJobMessage(String jobMessage) {
		this.jobMessage = jobMessage;
	}

	/**
	 * ハンドラーを実行します。
	 * このメソッドはメニューから呼び出されます。
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {

		// エディターをすべて閉じる（引数 true: 未保管の場合はダイアログ表示）
		UiPlugin.getActivePage().closeAllEditors(true);

		// 選択を取得
		final ISelection sel = HandlerUtil.getCurrentSelection(event);
		IStructuredSelection selection = StructuredSelection.EMPTY;
		if (sel instanceof IStructuredSelection) {
			selection = (IStructuredSelection) sel;
		}

		// ダイアログを作成 (テンプレート・メソッド)
		final Shell shell = HandlerUtil.getActiveShell(event);
		final AbstractWorkflowDialog dialog = (AbstractWorkflowDialog) createDialog(shell, selection);

		if (dialog != null && dialog.open() == Dialog.OK) {

			// ジョブ開始
			final WorkflowJob job = new WorkflowJob(jobMessage);
			job.setStatus(dialog.getStatus());
			job.setHandler(this);
			job.schedule();
		}
		return null;
	}

	/**
	 * ダイアログを作成します。
	 * @param shell シェル
	 * @param selection 選択
	 * @return ダイアログ
	 */
	abstract protected StatusDialog createDialog(Shell shell, IStructuredSelection selection);

	/**
	 * 処理を起動します。
	 * このメソッドは {@link WorkflowJob} から呼び出されます。
	 * @param iStatus ステータス
	 * @throws IOException 入出力例外が発生した場合
	 * @throws SAXException SAX 解析でエラーが発生した場合
	 */
	public void run(IStatus iStatus) throws IOException, SAXException {

		// ファイルを処理
		processFile(iStatus);

		if (monitor.isCanceled()) {
			return;
		}
		monitor.done();
	}

	/**
	 * ファイルを処理するメソッドです。
	 * @param iStatus ステータス (処理パラメーター)
	 * @throws IOException 入出力例外が発生した場合
	 * @throws SAXException SAX 解析でエラーが発生した場合
	 */
	abstract protected void processFile(IStatus iStatus) throws IOException, SAXException;

	/**
	 * キャンセルなしで正常終了した場合の処理です。
	 * デフォルトでは何も行いません。
	 * @param iStatus ステータス (処理パラメーター)
	 */
	protected void complete(IStatus iStatus) {
	}

	/**
	 * この処理の実行結果情報を取得します。
	 * @return 処理結果情報
	 */
	public BentenProcessResultInfo getResultInfo() {
		return resultInfo;
	}
}
